// main.cpp
//
// ICS 65 Fall 2012
// Code Example
//
// This source file exercises our Vector class in a variety of ways, so you
// can see the various effects of having the class be well-behaved.  Don't
// be bashful about writing your own main() function to replace mine with
// your own experiments; it'll take some experimentation to really understand
// what causes copy constructors, assignment operators, and destructors to be
// called, which is an important part of understanding this example.

#include <iostream>
using namespace std;

#include "Vector.h"


// This is a utility function that prints the contents of a Vector to the
// console, which I use throughout this source file.
void printAll(const Vector& v)
{
	cout << "Vector: size = " << v.size() << ", capacity = " << v.capacity() << endl;

	for (unsigned int i = 0; i < v.size(); i++)
	{
		cout << "    " << i << ": " << v.at(i) << endl;
	}
}


// This function accepts a vector by value.  Note that while we're allowed
// to make changes to it, those changes will not be reflected in the original.
void passingVectorByValue(Vector v)
{
	v.add("indeed");
	v.add("because");
	v.add("things");
	v.add("are");
	v.add("going");
	v.add("very");
	v.add("well");
	printAll(v);
}


// This function accepts a vector by reference, allowing us to make changes
// to it that will be reflected in the original.  Here, we introduce two
// changes: adding a new element to the end and changing an element using
// at() as an lvalue.
void passingVectorByReference(Vector& v)
{
	v.add("indeed");
	v.at(2) = "ecstatic";
	printAll(v);
}
// It may seem very strange that "v.at(2)" is allowed to be placed on the
// left-hand side of an assignment, but remember what this function returns:
// a reference to a string.  Given a reference to a string, you can assign a
// value into that string.  Since the Vector is returning a reference to one
// of the strings actually stored in its own array, assigning to v.at(2)
// actually changes the string in that array.


// This function demonstrates the effect of constness.  I've commented a
// few lines out that show things that are illegal to do to a const Vector,
// because they introduce changes to the Vector.
void theEffectOfConst(const Vector& v)
{
	cout << "theEffectOfConst" << endl;

	// We can't add to a const Vector, so this statement is illegal.
//	v.add("hello");

	// Because v is a const reference, calling v.at(3) is a call to the
	// const version of the function, which returns a const reference to a
	// string.  You can't assign to a const reference to a string (because
	// it represents a promise not to change that string), so this statement
	// is illegal.
//	v.at(3) = "Boo";

	// Reading a value using v.at(2) is fine, though.
	cout << v.at(2) << endl;

	// Copying v into a separate Vector, as well as modifying that copy, is
	// fine, because it doesn't change the original.
	Vector v2 = v;
	v2.add("hello");

	// Trying to acquire a non-const reference to v is problematic, though,
	// because that would introduce a way to change v, which we've promised
	// not to change.  So this statement is illegal.
//	Vector& v3 = v;

	printAll(v);
	printAll(v2);
}


int main()
{
	Vector v1;
	v1.add("Alex");
	v1.add("is");
	v1.add("happy");
	v1.add("today");
	printAll(v1);

	passingVectorByValue(v1);
	printAll(v1);

	passingVectorByReference(v1);
	printAll(v1);

	Vector v2(v1);
	v2.add("happily");

	Vector v3 = v1;
	v3.add("here");

	v1 = v3;

	printAll(v1);
	printAll(v2);
	printAll(v3);

	theEffectOfConst(v1);

	return 0;
}
